(function() {
  var elasticsearch;

  elasticsearch = require('elasticsearch');

  module.exports = function(RED) {
    var ElasticsearchNode;
    ElasticsearchNode = function(config) {
      var node;
      RED.nodes.createNode(this, config);
      node = this;
      return node.on("input", function(msg) {
        //console.log(config);
	var es;
	var es_hosts = [];
	var hosts = config.host.toString().split(',');
	var ports = config.port.toString().split(',');
	var index = config.index.toString();
	var documentType = config.documenttype.toString();
	var documentId = msg.es_documentId ? msg.es_documentId : null;
	var documentVersion =  msg.es_documentVersion ? msg.es_documentVersion : null ;
        var refreshIndex = !!msg.es_refresh; 
	var documentReplication = msg.es_replication && (msg.es_replication=="sync" || msg.es_replication=="async") ? msg.es_replication : "sync";
        var operationTimeout = !isNaN(msg.es_timeout) || (msg.es_timeout instanceof Date)  ? msg.es_timeout : 500 ;
	var documentTimeToLive = !isNaN(msg.es_ttl) ? msg.es_ttl : (59*60*24*365*100); //100 year

	//console.log(hosts,ports,index,documenttype);
	/*
		handle multiple hosts
	*/
	if(hosts.length==0 || hosts.length != ports.length){
		node.error('Wrong config, number of hosts & ports are different');
	}
	for(var i=0;i<hosts.length;i++){
		es_hosts.push({'host': hosts[i].toString(), 'port':ports[i]});
	}

	/*
		index & type 
		if index || type were provided by the msg object the use them
		else if daily_index == true then
			 index <= index + YYYYMMDD
		     	 type <= type
			else
	                 index <= index 
                         type <= type

	*/

	//Index
	if(msg.es_index){
		index = msg.es_index;
	}else{
		if(config.daily_index){
			var now = new Date();
			var yyyy = now.getFullYear().toString();
			var mm = (now.getMonth()+1).toString(); // getMonth() is zero-based
			var dd  = now.getDate().toString();
		        var yyyymmdd=  yyyy + (mm[1]?mm:"0"+mm[0]) + (dd[1]?dd:"0"+dd[0]); // padding
			index = index+yyyymmdd;
		}
	}

   	//document Type
	if(msg.es_documentType){
                documentType = msg.es_documentType;
        }

	//remove all es_* attributes
	for(var propertyName in msg) {
		//console.log("using "+propertyName);
		if(propertyName.substring(0,3)=="es_"){
			//console.log("going to delete "+propertyName);
			delete msg[propertyName];
		}
	}

	es = new elasticsearch.Client({
          host: es_hosts,
          log: 'error'
        });

        return es.create({
          index: index,
          type: documentType ,
          body: msg,
	  id : documentId,
	  version : documentVersion,
	  refresh : refreshIndex,
	  replication : documentReplication,
	  timeout : operationTimeout,
	  ttl : documentTimeToLive
        }, function(error, response) {
	  msg.es_error = false;
          if (error) {
	    	msg.es_error = true;
	    	msg.es_errorDescription = error;
	    	//            node.error(error, response);
          }else{
		msg.es_documentId = response.created ? response._id : null ;
		msg.es_documentVersion = response.created ? response._version : null ;
	  	msg.es_response = response;
	  	node.send(msg);
	  }
	  var es_retVal = es.close();
	  return es_retVal;
        });
	//curl '$host:$port/_nodes/stats/http?pretty'
	delete es; //leak without delete ? 
      });
    };
    return RED.nodes.registerType("elasticsearch2", ElasticsearchNode);
  };

}).call(this);
